"use strict";

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = GridItem;

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _clsx = require("clsx");

var _clsx2 = _interopRequireDefault(_clsx);

var _Draggable = require("./Draggable");

var _Draggable2 = _interopRequireDefault(_Draggable);

var _Resizable = require("./Resizable");

var _Resizable2 = _interopRequireDefault(_Resizable);

var _propTypes3 = require("./propTypes");

var propTypes = _interopRequireWildcard(_propTypes3);

var _utils = require("./utils");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/**
 * An individual item within a ReactGridLayout.
 */
function GridItem(_ref) {
  var containerPadding = _ref.containerPadding,
      margin = _ref.margin,
      rowHeight = _ref.rowHeight,
      props = _objectWithoutProperties(_ref, ["containerPadding", "margin", "rowHeight"]);

  var _useState = (0, _react.useState)(null),
      resizing = _useState[0],
      setResizing = _useState[1];

  var _useState2 = (0, _react.useState)(null),
      dragging = _useState2[0],
      setDragging = _useState2[1];

  // Helper for generating column width


  function calcColWidth() {
    var containerWidth = props.containerWidth,
        cols = props.cols;

    return (containerWidth - margin[0] * (cols - 1) - containerPadding[0] * 2) / cols;
  }

  /**
   * Return position on the page given an x, y, w, h.
   * left, top, width, height are all in pixels.
   * @param  {Number}  x             X coordinate in grid units.
   * @param  {Number}  y             Y coordinate in grid units.
   * @param  {Number}  w             W coordinate in grid units.
   * @param  {Number}  h             H coordinate in grid units.
   * @return {Object}                Object containing coords.
   */
  function calcPosition(x, y, w, h, state) {
    var colWidth = calcColWidth();
    var out = {};

    // If resizing, use the exact width and height as returned from resizing callbacks.
    if (state && state.resizing) {
      out.width = Math.round(state.resizing.width);
      out.height = Math.round(state.resizing.height);
    }
    // Otherwise, calculate from grid units.
    else {
        // 0 * Infinity === NaN, which causes problems with resize constraints;
        // Fix this if it occurs.
        // Note we do it here rather than later because Math.round(Infinity) causes deopt
        out.width = w === Infinity ? w : calcWidth(w, colWidth);
        out.height = h === Infinity ? h : calcHeight(h);
      }

    if (state && state.dragging) {
      out.top = Math.round(state.dragging.top);
      out.left = Math.round(state.dragging.left);
    }
    // Otherwise, calculate from grid units.
    // TODO: unnecessary vvv
    else {
        out.top = Math.round((rowHeight + margin[1]) * y + containerPadding[1]);
        out.left = Math.round((colWidth + margin[0]) * x + containerPadding[0]);
      }

    return out;
  }

  /**
   * Translate x and y coordinates from pixels to grid units.
   * @param  {Number} top  Top position (relative to parent) in pixels.
   * @param  {Number} left Left position (relative to parent) in pixels.
   * @return {Object} x and y in grid units.
   */
  function calcXY(top, left) {
    var cols = props.cols,
        w = props.w,
        h = props.h,
        maxRows = props.maxRows;

    var colWidth = calcColWidth();

    // left = colWidth * x + margin * (x + 1)
    // l = cx + m(x+1)
    // l = cx + mx + m
    // l - m = cx + mx
    // l - m = x(c + m)
    // (l - m) / (c + m) = x
    // x = (left - margin) / (coldWidth + margin)
    var x = Math.round((left - containerPadding[0]) / (colWidth + margin[0]));
    var y = Math.round((top - containerPadding[1]) / (rowHeight + margin[1]));

    // Capping
    x = Math.max(Math.min(x, cols - w), 0);
    y = Math.max(Math.min(y, maxRows - h), 0);

    return { x: x, y: y };
  }

  /**
   * Given a height and width in pixel values, calculate grid units.
   * @param  {Number} height Height in pixels.
   * @param  {Number} width  Width in pixels.
   * @return {Object} w, h as grid units.
   */
  function calcWH(_ref2) {
    var height = _ref2.height,
        width = _ref2.width;
    var maxRows = props.maxRows,
        cols = props.cols,
        x = props.x,
        y = props.y;

    var colWidth = calcColWidth();

    // width = colWidth * w - (margin * (w - 1))
    // ...
    // w = (width + margin) / (colWidth + margin)
    var w = Math.round((width + margin[0]) / (colWidth + margin[0]));
    var h = Math.round((height + margin[1]) / (rowHeight + margin[1]));

    // Capping
    w = Math.max(Math.min(w, cols - x), 0);
    h = Math.max(Math.min(h, maxRows - y), 0);
    return { w: w, h: h };
  }

  /**
   * Calculate grid item width
   * @param  {Number} w        W coordinate in grid units
   * @param  {Number} colWidth Column width in pixels
   * @return {Number} Item width in pixels
   */
  function calcWidth(w, colWidth) {
    return Math.round(colWidth * w + Math.max(0, w - 1) * margin[0]);
  }

  /**
   * Calculate grid item height
   * @param  {Number} h H coordinate in grid units
   * @return {Number} Item height in pixels
   */
  function calcHeight(h) {
    return Math.round(rowHeight * h + Math.max(0, h - 1) * margin[1]);
  }

  /**
   * This is where we set the grid item's absolute placement. It gets a little tricky because we want to do it
   * well when server rendering, and the only way to do that properly is to use percentage width/left because
   * we don't know exactly what the browser viewport is.
   * Unfortunately, CSS Transforms, which are great for performance, break in this instance because a percentage
   * left is relative to the item itself, not its container! So we cannot use them on the server rendering pass.
   *
   * @param  {Object} pos Position object with width, height, left, top.
   * @return {Object}     Style object.
   */
  function createStyle(pos) {
    var usePercentages = props.usePercentages,
        containerWidth = props.containerWidth,
        useCSSTransforms = props.useCSSTransforms;

    var style = void 0;
    // CSS Transforms support (default)
    if (useCSSTransforms) {
      style = (0, _utils.setTransform)(pos);
    } else {
      // top,left (slow)
      style = (0, _utils.setTopLeft)(pos);

      // This is used for server rendering.
      if (usePercentages) {
        style.left = (0, _utils.perc)(pos.left / containerWidth);
        style.width = (0, _utils.perc)(pos.width / containerWidth);
      }
    }

    return style;
  }

  /**
   * Wrapper around drag events to provide more useful data.
   * All drag events call the function with the given handler name,
   * with the signature (index, x, y).
   * @param e
   * @param node
   * @param deltaX
   * @param deltaY
   * @param {String} handlerName - Handler name to wrap.
   * @return {*}
   */
  function onDragHandler(e, _ref3, handlerName) {
    var node = _ref3.node,
        deltaX = _ref3.deltaX,
        deltaY = _ref3.deltaY;

    if (!props[handlerName]) return;

    var newPosition = { top: 0, left: 0 };

    // Get new XY
    switch (handlerName) {
      case "onDragStart":
        {
          // Stops bubbling to allow nested grid dragging
          e.stopPropagation();

          // TODO: this wont work on nested parents
          var offsetParent = node.offsetParent;

          if (!offsetParent) return;
          var parentRect = offsetParent.getBoundingClientRect();
          var clientRect = node.getBoundingClientRect();
          newPosition.left = clientRect.left - parentRect.left + offsetParent.scrollLeft;
          newPosition.top = clientRect.top - parentRect.top + offsetParent.scrollTop;
          setDragging(newPosition);
          break;
        }
      case "onDrag":
        {
          if (!dragging) throw new Error("onDrag called before onDragStart.");
          var top = dragging.top + deltaY;
          newPosition.top = dragging.top + deltaY;
          var left = dragging.left + deltaX;

          var isBounded = props.isBounded,
              _w = props.w,
              _h = props.h,
              containerWidth = props.containerWidth;


          if (isBounded) {
            var _offsetParent = node.offsetParent;


            if (_offsetParent) {
              var bottomBoundary = _offsetParent.clientHeight - calcHeight(_h);
              if (top > bottomBoundary) top = bottomBoundary;
              if (top < 0) top = 0;

              var rightBoundary = containerWidth - calcWidth(_w, calcColWidth());
              if (left > rightBoundary) left = rightBoundary;
              if (left < 0) left = 0;
            }
          }

          newPosition.left = left;
          newPosition.top = top;

          setDragging(newPosition);
          break;
        }
      case "onDragStop":
        if (!dragging) throw new Error("onDragEnd called before onDragStart.");
        newPosition.left = dragging.left;
        newPosition.top = dragging.top;
        setDragging(null);
        break;
      default:
        throw new Error("onDragHandler called with unrecognized handlerName: " + handlerName);
    }

    var _calcXY = calcXY(newPosition.top, newPosition.left),
        x = _calcXY.x,
        y = _calcXY.y;

    return props[handlerName].call(this, props.i, x, y, { e: e, node: node, newPosition: newPosition });
  }

  /**
   * Wrapper around drag events to provide more useful data.
   * All drag events call the function with the given handler name,
   * with the signature (index, x, y).
   * @param e
   * @param node
   * @param size
   * @param {String} handlerName - Handler name to wrap.
   */
  function onResizeHandler(e, _ref4, handlerName) {
    var node = _ref4.node,
        size = _ref4.size;

    if (!props[handlerName]) return;
    var cols = props.cols,
        x = props.x,
        i = props.i,
        maxW = props.maxW,
        minW = props.minW,
        maxH = props.maxH,
        minH = props.minH;

    // Get new XY

    var _calcWH = calcWH(size),
        w = _calcWH.w,
        h = _calcWH.h;

    // Cap w at numCols


    w = Math.min(w, cols - x);
    // Ensure w is at least 1
    w = Math.max(w, 1);

    // Min/max capping
    w = Math.max(Math.min(w, maxW), minW);
    h = Math.max(Math.min(h, maxH), minH);

    setResizing(handlerName === "onResizeStop" ? null : size);

    props[handlerName](i, w, h, { e: e, node: node, size: size });
  }

  var cancel = props.cancel,
      handle = props.handle,
      cols = props.cols,
      x = props.x,
      y = props.y,
      w = props.w,
      h = props.h,
      minW = props.minW,
      minH = props.minH,
      maxW = props.maxW,
      maxH = props.maxH,
      isDraggable = props.isDraggable,
      isResizable = props.isResizable,
      useCSSTransforms = props.useCSSTransforms,
      resizableProps = props.resizableProps,
      style = props.style;


  var pos = calcPosition(x, y, w, h, { dragging: dragging, resizing: resizing });
  var child = _react2.default.Children.only(props.children);

  // Create the child element. We clone the existing element but modify its className and style.
  var newChild = _react2.default.cloneElement(child, {
    className: (0, _clsx2.default)("react-grid-item", child.props.className, props.className, {
      static: props.static,
      resizing: Boolean(resizing),
      "react-draggable": isDraggable,
      "react-draggable-dragging": Boolean(dragging),
      cssTransforms: useCSSTransforms
    }),
    // We can set the width and height on the child, but unfortunately we can't set the position.
    style: _extends({}, style, child.props.style, createStyle(pos))
  });

  // `data-grid` isResizable can override any other props
  var resizable = child.props["data-grid"] && typeof child.props["data-grid"].isResizable === "boolean" ? child.props["data-grid"].isResizable : isResizable;

  return _react2.default.createElement(
    _Draggable2.default,
    {
      cancel: cancel,
      handle: handle,
      isDraggable: isDraggable,
      isResizable: resizable,
      onDrag: onDragHandler
    },
    _react2.default.createElement(
      _Resizable2.default,
      {
        calcPosition: calcPosition,
        cols: cols,
        x: x,
        minW: minW,
        minH: minH,
        maxW: maxW,
        maxH: maxH,
        isResizable: resizable,
        onResize: onResizeHandler,
        position: pos,
        resizableProps: resizableProps
      },
      newChild
    )
  );
}

GridItem.propTypes = {
  // Children must be only a single element
  children: _propTypes2.default.element,
  style: _propTypes2.default.object,

  // General grid attributes
  cols: _propTypes2.default.number.isRequired,
  containerWidth: _propTypes2.default.number.isRequired,
  rowHeight: _propTypes2.default.number.isRequired,
  margin: _propTypes2.default.array.isRequired,
  maxRows: _propTypes2.default.number.isRequired,
  containerPadding: _propTypes2.default.array.isRequired,

  // These are all in grid units
  x: _propTypes2.default.number.isRequired,
  y: _propTypes2.default.number.isRequired,
  w: _propTypes2.default.number.isRequired,
  h: _propTypes2.default.number.isRequired,

  // All optional
  minW: propTypes.minW,
  maxW: propTypes.maxW,
  minH: propTypes.minH,
  maxH: propTypes.maxH,

  // ID is nice to have for callbacks
  i: _propTypes2.default.string.isRequired,

  // Functions
  onDragStop: _propTypes2.default.func,
  onDragStart: _propTypes2.default.func,
  onDrag: _propTypes2.default.func,
  onResizeStop: _propTypes2.default.func,
  onResizeStart: _propTypes2.default.func,
  onResize: _propTypes2.default.func,

  // Flags
  isDraggable: _propTypes2.default.bool.isRequired,
  isResizable: _propTypes2.default.bool.isRequired,
  isBounded: _propTypes2.default.bool.isRequired,
  static: _propTypes2.default.bool,

  // Use CSS transforms instead of top/left
  useCSSTransforms: _propTypes2.default.bool.isRequired,

  resizableProps: _propTypes2.default.object.isRequired,

  // Others
  className: _propTypes2.default.string,
  // Selector for draggable handle
  handle: _propTypes2.default.string,
  // Selector for draggable cancel (see react-draggable)
  cancel: _propTypes2.default.string,
  usePercentages: _propTypes2.default.bool
};

GridItem.defaultProps = {
  className: "",
  cancel: "",
  handle: "",
  minH: 1,
  minW: 1,
  maxH: Infinity,
  maxW: Infinity,
  usePercentages: false
};