"use strict";

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = ReactGridLayout;

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require("lodash.isequal");

var _lodash2 = _interopRequireDefault(_lodash);

var _clsx = require("clsx");

var _clsx2 = _interopRequireDefault(_clsx);

var _utils = require("./utils");

var _GridItem = require("./GridItem");

var _GridItem2 = _interopRequireDefault(_GridItem);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// End Types

/**
 * A reactive, fluid grid layout with draggable, resizable components.
 */

// Types
function ReactGridLayout(_ref) {
  var _ref$autoSize = _ref.autoSize,
      autoSize = _ref$autoSize === undefined ? true : _ref$autoSize,
      children = _ref.children,
      _ref$className = _ref.className,
      className = _ref$className === undefined ? "" : _ref$className,
      _ref$cols = _ref.cols,
      cols = _ref$cols === undefined ? 12 : _ref$cols,
      _ref$compactType = _ref.compactType,
      compactType = _ref$compactType === undefined ? "vertical" : _ref$compactType,
      _ref$containerPadding = _ref.containerPadding,
      containerPadding = _ref$containerPadding === undefined ? null : _ref$containerPadding,
      _ref$draggableCancel = _ref.draggableCancel,
      draggableCancel = _ref$draggableCancel === undefined ? "" : _ref$draggableCancel,
      _ref$draggableHandle = _ref.draggableHandle,
      draggableHandle = _ref$draggableHandle === undefined ? "" : _ref$draggableHandle,
      _ref$isBounded = _ref.isBounded,
      isBounded = _ref$isBounded === undefined ? false : _ref$isBounded,
      _ref$isDraggable = _ref.isDraggable,
      isDraggable = _ref$isDraggable === undefined ? true : _ref$isDraggable,
      _ref$isResizable = _ref.isResizable,
      isResizable = _ref$isResizable === undefined ? true : _ref$isResizable,
      _ref$layout = _ref.layout,
      layout = _ref$layout === undefined ? [] : _ref$layout,
      _ref$margin = _ref.margin,
      margin = _ref$margin === undefined ? [10, 10] : _ref$margin,
      _ref$maxRows = _ref.maxRows,
      maxRows = _ref$maxRows === undefined ? Infinity : _ref$maxRows,
      _ref$onDrag = _ref.onDrag,
      onDrag = _ref$onDrag === undefined ? _utils.noop : _ref$onDrag,
      _ref$onDragStart = _ref.onDragStart,
      onDragStart = _ref$onDragStart === undefined ? _utils.noop : _ref$onDragStart,
      _ref$onDragStop = _ref.onDragStop,
      onDragStop = _ref$onDragStop === undefined ? _utils.noop : _ref$onDragStop,
      _ref$onLayoutChange = _ref.onLayoutChange,
      onLayoutChange = _ref$onLayoutChange === undefined ? _utils.noop : _ref$onLayoutChange,
      _ref$onResize = _ref.onResize,
      onResize = _ref$onResize === undefined ? _utils.noop : _ref$onResize,
      _ref$onResizeStart = _ref.onResizeStart,
      onResizeStart = _ref$onResizeStart === undefined ? _utils.noop : _ref$onResizeStart,
      _ref$onResizeStop = _ref.onResizeStop,
      onResizeStop = _ref$onResizeStop === undefined ? _utils.noop : _ref$onResizeStop,
      _ref$preventCollision = _ref.preventCollision,
      preventCollision = _ref$preventCollision === undefined ? false : _ref$preventCollision,
      _ref$resizableProps = _ref.resizableProps,
      resizableProps = _ref$resizableProps === undefined ? {} : _ref$resizableProps,
      _ref$rowHeight = _ref.rowHeight,
      rowHeight = _ref$rowHeight === undefined ? 150 : _ref$rowHeight,
      _ref$style = _ref.style,
      style = _ref$style === undefined ? {} : _ref$style,
      _ref$useCSSTransforms = _ref.useCSSTransforms,
      useCSSTransforms = _ref$useCSSTransforms === undefined ? true : _ref$useCSSTransforms,
      _ref$verticalCompact = _ref.verticalCompact,
      verticalCompact = _ref$verticalCompact === undefined ? true : _ref$verticalCompact,
      width = _ref.width;

  var _useState = (0, _react.useState)(null),
      activeDrag = _useState[0],
      setActiveDrag = _useState[1];

  var _useState2 = (0, _react.useState)((0, _utils.synchronizeLayoutWithChildren)(layout, children, cols,
  // Legacy support for verticalCompact: false
  fixCompactType())),
      layoutState = _useState2[0],
      setLayoutState = _useState2[1];

  var _useState3 = (0, _react.useState)(false),
      mounted = _useState3[0],
      setMounted = _useState3[1];

  var _useState4 = (0, _react.useState)(null),
      oldDragItem = _useState4[0],
      setOldDragItem = _useState4[1];

  var _useState5 = (0, _react.useState)(null),
      oldLayout = _useState5[0],
      setOldLayout = _useState5[1];

  var _useState6 = (0, _react.useState)(null),
      oldResizeItem = _useState6[0],
      setOldResizeItem = _useState6[1];

  var _useState7 = (0, _react.useState)([]),
      childrenKeys = _useState7[0],
      setChildrenKeys = _useState7[1];

  (0, _react.useEffect)(function () {
    setMounted(true);
    // Possibly call back with layout on mount. This should be done after
    // correcting the layout width to ensure we don't rerender with the wrong
    // width.
    handleLayoutMaybeChanged(layout, [].concat(layoutState));
  }, []);

  (0, _react.useEffect)(function () {
    setNewLayout(layout);
  }, [layout, compactType]);

  (0, _react.useEffect)(function () {
    var currentChildrenKeys = (0, _utils.getChildrenKeys)(children);
    // If children change, also regenerate the layout. Use our state
    // as the base in case because it may be more up to date than
    // what is in props.
    if (!(0, _lodash2.default)(currentChildrenKeys, childrenKeys)) {
      setNewLayout(layout);
      setChildrenKeys(currentChildrenKeys);
    }
  });

  function setNewLayout(newLayoutBase) {
    var newLayout = (0, _utils.synchronizeLayoutWithChildren)(newLayoutBase, children, cols, fixCompactType());

    var old = [].concat(layoutState);
    setLayoutState(newLayout);
    handleLayoutMaybeChanged(newLayout, old);
  }

  // componentWillReceiveProps(nextProps: Props) {
  //   const { layout } = this.state;
  //   let newLayoutBase;
  //   // Legacy support for compactType
  //   // Allow parent to set layout directly.
  //   if (
  //     !isEqual(nextProps.layout, this.props.layout) ||
  //     nextProps.compactType !== this.props.compactType
  //   ) {
  //     newLayoutBase = nextProps.layout;
  //   } else if (!childrenEqual(this.props.children, nextProps.children)) {
  //     // If children change, also regenerate the layout. Use our state
  //     // as the base in case because it may be more up to date than
  //     // what is in props.
  //     newLayoutBase = layout;
  //   }
  //
  //   // We need to regenerate the layout.
  //   if (newLayoutBase) {
  //     // allow to initialize layout before the first children will be added
  //     // this is handy for async items add, when we can't use per-item layout
  //     const newLayout = nextProps.children
  //       ? synchronizeLayoutWithChildren(
  //           newLayoutBase,
  //           nextProps.children,
  //           nextProps.cols,
  //           this.compactType(nextProps)
  //         )
  //       : newLayoutBase;
  //     this.setState({ layout: newLayout });
  //     onLayoutMaybeChanged(newLayout, layout);
  //   }
  // }

  /**
   * Calculates a pixel value for the container.
   * @return {String} Container height in pixels.
   */
  function containerHeight() {
    if (!autoSize) {
      return;
    }
    var nbRow = (0, _utils.bottom)(layoutState);
    var containerPaddingY = containerPadding ? containerPadding[1] : margin[1];
    return nbRow * rowHeight + (nbRow - 1) * margin[1] + containerPaddingY * 2 + "px";
  }

  function fixCompactType() {
    return verticalCompact === false ? null : compactType;
  }

  /**
   * When dragging starts
   * @param {String} i Id of the child
   * @param {Number} x X position of the move
   * @param {Number} y Y position of the move
   * @param {Event} e The mousedown event
   * @param {Element} node The current dragging DOM element
   */
  function handleDragStart(i, x, y, _ref2) {
    var e = _ref2.e,
        node = _ref2.node;

    var l = (0, _utils.getLayoutItem)(layoutState, i);

    if (!l) {
      return;
    }

    setOldDragItem((0, _utils.cloneLayoutItem)(l));
    setOldLayout(layoutState);

    return onDragStart(layoutState, l, l, null, e, node);
  }

  /**
   * Each drag movement create a new dragelement and move the element to the dragged location
   * @param {String} i Id of the child
   * @param {Number} x X position of the move
   * @param {Number} y Y position of the move
   * @param {Event} e The mousedown event
   * @param {Element} node The current dragging DOM element
   */
  function handleDrag(i, x, y, _ref3) {
    var e = _ref3.e,
        node = _ref3.node;

    var l = (0, _utils.getLayoutItem)(layoutState, i);

    if (!l) {
      return;
    }

    // Create placeholder (display only)
    var placeholder = {
      w: l.w,
      h: l.h,
      x: l.x,
      y: l.y,
      placeholder: true,
      i: i
    };

    var modifiedLayout = (0, _utils.moveElement)([].concat(layoutState), l, x, y,
    // Move the element to the dragged location.
    true, preventCollision, fixCompactType(), cols);

    onDrag(modifiedLayout, oldDragItem, l, placeholder, e, node);

    setLayoutState((0, _utils.compact)(modifiedLayout, fixCompactType(), cols));
    setActiveDrag(placeholder);
  }

  /**
   * When dragging stops, figure out which position the element is closest to and update its x and y.
   * @param  {String} i Index of the child.
   * @param {Number} x X position of the move
   * @param {Number} y Y position of the move
   * @param {Event} e The mousedown event
   * @param {Element} node The current dragging DOM element
   */
  function handleDragStop(i, x, y, _ref4) {
    var e = _ref4.e,
        node = _ref4.node;

    var l = (0, _utils.getLayoutItem)(layoutState, i);

    if (!l) {
      return;
    }

    var modifiedLayout = (0, _utils.moveElement)([].concat(layoutState), l, x, y,
    // Move the element here
    true, preventCollision, fixCompactType(), cols);

    onDragStop(modifiedLayout, oldDragItem, l, null, e, node);

    var newLayout = (0, _utils.compact)(modifiedLayout, fixCompactType(), cols);
    var old = [].concat(layoutState);

    setActiveDrag(null);
    setLayoutState(newLayout);
    setOldDragItem(null);
    setOldLayout(null);

    handleLayoutMaybeChanged(newLayout, old);
  }

  function handleLayoutMaybeChanged(newLayoutState, oldLayoutState) {
    var old = oldLayoutState ? oldLayoutState : layoutState;
    if (!(0, _lodash2.default)(old, newLayoutState)) {
      onLayoutChange(newLayoutState);
    }
  }

  function handleResizeStart(i, w, h, _ref5) {
    var e = _ref5.e,
        node = _ref5.node;

    var l = (0, _utils.getLayoutItem)(layoutState, i);

    if (!l) {
      return;
    }

    setOldResizeItem((0, _utils.cloneLayoutItem)(l));
    // thanks https://github.com/STRML/react-grid-layout/pull/1219
    setOldLayout(layoutState.map(function (layoutItem) {
      return (0, _utils.cloneLayoutItem)(layoutItem);
    }));

    onResizeStart(layoutState, l, l, null, e, node);
  }

  function handleResize(i, w, h, _ref6) {
    var e = _ref6.e,
        node = _ref6.node;

    var l = (0, _utils.getLayoutItem)(layoutState, i);

    if (!l) {
      return;
    }

    // Something like quad tree should be used
    // to find collisions faster
    var hasCollisions = false;

    if (preventCollision) {
      var collisions = (0, _utils.getAllCollisions)(layoutState, _extends({}, l, {
        w: w,
        h: h
      })).filter(function (layoutItem) {
        return layoutItem.i !== l.i;
      });
      hasCollisions = collisions.length > 0;

      // If we're colliding, we need adjust the placeholder.
      if (hasCollisions) {
        // adjust w && h to maximum allowed space
        var leastX = Infinity;
        var leastY = Infinity;

        collisions.forEach(function (layoutItem) {
          if (layoutItem.x > l.x) {
            leastX = Math.min(leastX, layoutItem.x);
          }
          if (layoutItem.y > l.y) {
            leastY = Math.min(leastY, layoutItem.y);
          }
        });

        if (Number.isFinite(leastX)) {
          l.w = leastX - l.x;
        }
        if (Number.isFinite(leastY)) {
          l.h = leastY - l.y;
        }
      }
    }

    if (!hasCollisions) {
      // Set new width and height.
      l.w = w;
      l.h = h;
      l.persistentW = w;
      l.persistentH = h;
    }

    // Create placeholder element (display only)
    var placeholder = {
      w: l.w,
      h: l.h,
      x: l.x,
      y: l.y,
      static: true,
      i: i
    };

    onResize(layoutState, oldResizeItem, l, placeholder, e, node);

    // Re-compact the layout and set the drag placeholder.
    setLayoutState((0, _utils.compact)([].concat(layoutState), fixCompactType(), cols));
    setActiveDrag(placeholder);
  }

  function handleResizeStop(i, w, h, _ref7) {
    var e = _ref7.e,
        node = _ref7.node;

    var l = (0, _utils.getLayoutItem)(layoutState, i);

    onResizeStop(layoutState, oldResizeItem, l, null, e, node);

    // Set state
    var newLayout = (0, _utils.compact)(layoutState, fixCompactType(), cols);

    setActiveDrag(null);
    setLayoutState(newLayout);
    setOldResizeItem(null);
    setOldLayout(null);

    handleLayoutMaybeChanged(newLayout, oldLayout);
  }

  /**
   * Create a placeholder object.
   * @return {Element} Placeholder div.
   */
  function placeholder() {
    if (!activeDrag) {
      return null;
    }

    // {...this.state.activeDrag} is pretty slow, actually
    return _react2.default.createElement(
      _GridItem2.default,
      {
        w: activeDrag.w,
        h: activeDrag.h,
        x: activeDrag.x,
        y: activeDrag.y,
        i: activeDrag.i,
        className: "react-grid-placeholder",
        containerWidth: width,
        cols: cols,
        margin: margin,
        containerPadding: containerPadding || margin,
        maxRows: maxRows,
        rowHeight: rowHeight,
        isDraggable: false,
        isResizable: false,
        isBounded: false,
        useCSSTransforms: useCSSTransforms,
        resizableProps: resizableProps
      },
      _react2.default.createElement("div", null)
    );
  }

  /**
   * Given a grid item, set its style attributes & surround in a <Draggable>.
   * @param  {Element} child React element.
   * @return {Element}       Element wrapped in draggable and properly placed.
   */
  function processGridItem(child) {
    if (!child || !child.key) {
      return;
    }

    var l = (0, _utils.getLayoutItem)(layoutState, String(child.key));
    if (!l) {
      return null;
    }

    // Parse 'static'. Any properties defined directly on the grid item will take precedence.
    var draggable = Boolean(!l.static && isDraggable && (l.isDraggable || l.isDraggable == null));
    var resizable = Boolean(!l.static && isResizable && (l.isResizable || l.isResizable == null));

    var bounded = Boolean(draggable && isBounded && (l.isBounded || l.isBounded == null));

    return _react2.default.createElement(
      _GridItem2.default,
      {
        containerWidth: width,
        cols: cols,
        margin: margin,
        containerPadding: containerPadding || margin,
        maxRows: maxRows,
        rowHeight: rowHeight,
        cancel: draggableCancel,
        handle: draggableHandle,
        onDragStop: handleDragStop,
        onDragStart: handleDragStart,
        onDrag: handleDrag,
        onResizeStart: handleResizeStart,
        onResize: handleResize,
        onResizeStop: handleResizeStop,
        isDraggable: draggable,
        isResizable: resizable,
        isBounded: bounded,
        useCSSTransforms: useCSSTransforms && mounted,
        usePercentages: !mounted,
        resizableProps: resizableProps,
        w: l.w,
        h: l.h,
        x: l.x,
        y: l.y,
        i: l.i,
        persistentH: l.persistentH,
        persistentW: l.persistentW,
        minH: l.minH,
        minW: l.minW,
        maxH: l.maxH,
        maxW: l.maxW,
        "static": l.static
      },
      child
    );
  }

  var mergedClassName = (0, _clsx2.default)("react-grid-layout", className);
  var mergedStyle = _extends({
    height: containerHeight()
  }, style);

  return _react2.default.createElement(
    "div",
    { className: mergedClassName, style: mergedStyle },
    _react2.default.Children.map(children, function (child) {
      return processGridItem(child);
    }),
    placeholder()
  );
}

ReactGridLayout.displayName = "ReactGridLayout";

ReactGridLayout.propTypes = {
  //
  // Basic props
  //
  className: _propTypes2.default.string,
  style: _propTypes2.default.object,

  // This can be set explicitly. If it is not set, it will automatically
  // be set to the container width. Note that resizes will *not* cause this to adjust.
  // If you need that behavior, use WidthProvider.
  width: _propTypes2.default.number,

  // If true, the container height swells and contracts to fit contents
  autoSize: _propTypes2.default.bool,
  // # of cols.
  cols: _propTypes2.default.number,

  // A selector that will not be draggable.
  draggableCancel: _propTypes2.default.string,
  // A selector for the draggable handler
  draggableHandle: _propTypes2.default.string,

  // Deprecated
  verticalCompact: function verticalCompact(props) {
    if (props.verticalCompact === false && process.env.NODE_ENV !== "production") {
      console.warn(
      // eslint-disable-line no-console
      "`verticalCompact` on <ReactGridLayout> is deprecated and will be removed soon. " + 'Use `compactType`: "horizontal" | "vertical" | null.');
    }
  },
  // Choose vertical or hotizontal compaction
  compactType: _propTypes2.default.oneOf(["vertical", "horizontal"]),

  // layout is an array of object with the format:
  // {x: Number, y: Number, w: Number, h: Number, i: String}
  layout: function layout(props) {
    var layout = props.layout;
    // I hope you're setting the data-grid property on the grid items
    if (layout === undefined) return;
    (0, _utils.validateLayout)(layout, "layout");
  },

  //
  // Grid Dimensions
  //

  // Margin between items [x, y] in px
  margin: _propTypes2.default.arrayOf(_propTypes2.default.number),
  // Padding inside the container [x, y] in px
  containerPadding: _propTypes2.default.arrayOf(_propTypes2.default.number),
  // Rows have a static height, but you can change this based on breakpoints if you like
  rowHeight: _propTypes2.default.number,
  // Default Infinity, but you can specify a max here if you like.
  // Note that this isn't fully fleshed out and won't error if you specify a layout that
  // extends beyond the row capacity. It will, however, not allow users to drag/resize
  // an item past the barrier. They can push items beyond the barrier, though.
  // Intentionally not documented for this reason.
  maxRows: _propTypes2.default.number,

  //
  // Flags
  //
  isDraggable: _propTypes2.default.bool,
  isResizable: _propTypes2.default.bool,
  isBounded: _propTypes2.default.bool,
  // If true, grid items won't change position when being dragged over.
  preventCollision: _propTypes2.default.bool,
  // Use CSS transforms instead of top/left
  useCSSTransforms: _propTypes2.default.bool,
  // Additional props for React-Resizable
  resizableProps: _propTypes2.default.object,

  //
  // Callbacks
  //

  // Callback so you can save the layout. Calls after each drag & resize stops.
  onLayoutChange: _propTypes2.default.func,

  // Calls when drag starts. Callback is of the signature (layout, oldItem, newItem, placeholder, e, ?node).
  // All callbacks below have the same signature. 'start' and 'stop' callbacks omit the 'placeholder'.
  onDragStart: _propTypes2.default.func,
  // Calls on each drag movement.
  onDrag: _propTypes2.default.func,
  // Calls when drag is complete.
  onDragStop: _propTypes2.default.func,
  //Calls when resize starts.
  onResizeStart: _propTypes2.default.func,
  // Calls when resize movement happens.
  onResize: _propTypes2.default.func,
  // Calls when resize is complete.
  onResizeStop: _propTypes2.default.func,

  //
  // Other validations
  //

  // Children must not have duplicate keys.
  children: function children(props, propName) {
    var children = props[propName];

    // Check children keys for duplicates. Throw if found.
    var keys = {};
    _react2.default.Children.forEach(children, function (child) {
      if (keys[child.key]) {
        throw new Error('Duplicate child key "' + child.key + '" found! This will cause problems in ReactGridLayout.');
      }
      keys[child.key] = true;
    });
  }
};